import importlib.metadata

try:
    __version__ = importlib.metadata.version(__name__)
except importlib.metadata.PackageNotFoundError:
    # package is not installed
    pass


def _walk_obj(obj, func, leaf_condition=None):
    """Walk through a nested object applying func to each leaf element.

    This function returns a recursively built structure from a nested
    tree of simple container types (e.g. dict, list, tuple) by
    applying func to each leaf node in the tree. By default a leaf
    node is considered to be any object that is not a dict, list or
    tuple.

    leaf_condition may be used if certain types of lists, tuples or
    dicts should be considered leaf nodes. leaf_condition should be
    passed a function that takes a sub-tree and returns True if that
    sub-tree is a leaf node, or False if _walk_obj should continue to
    descend through the sub-tree.

    """
    if hasattr(leaf_condition, '__call__'):
        if leaf_condition(obj):
            return func(obj)

    if isinstance(obj, dict):
        return {k: _walk_obj(v, func, leaf_condition=leaf_condition)
                for k, v in obj.items()}

    elif isinstance(obj, list):
        return [_walk_obj(v, func, leaf_condition=leaf_condition) for v in obj]

    elif isinstance(obj, tuple):
        return tuple(_walk_obj(list(obj), func, leaf_condition=leaf_condition))

    return func(obj)



from kombu.exceptions import DecodeError

from girder_worker_utils.transform import Transform


class Capitalize(Transform):
    def transform(self):
        return self.value.upper()


class RaiseException(Transform):

    def transform(self):
        raise DecodeError('Some kind of Error')


class Reverse(Transform):
    def transform(self):
        return self.value[::-1]



import os

from setuptools import find_packages, setup

with open('requirements.in') as f:
    install_requires = f.readlines()


def prerelease_local_scheme(version):
    """Return local scheme version unless building on master in CircleCI.

    This function returns the local scheme version number
    (e.g. 0.0.0.dev<N>+g<HASH>) unless building on CircleCI for a
    pre-release in which case it ignores the hash and produces a
    PEP440 compliant pre-release version number (e.g. 0.0.0.dev<N>).
    """
    from setuptools_scm.version import get_local_node_and_date

    if 'CIRCLE_BRANCH' in os.environ and \
       os.environ['CIRCLE_BRANCH'] == 'master':
        return ''
    else:
        return get_local_node_and_date(version)


setup(
    name='girder-worker-utils',
    use_scm_version={'local_scheme': prerelease_local_scheme},
    setup_requires=['setuptools_scm'],
    description='Helper utilities for the Girder Worker',
    author='Kitware, Inc.',
    author_email='kitware@kitware.com',
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    ],
    install_requires=install_requires,
    python_requires='>=3.8',
    packages=find_packages(
        exclude=('tests.*', 'tests')
    ),
    zip_safe=False
)



import functools

from celery import Celery, Task
from kombu.serialization import register
from kombu.utils import json

from girder_worker_utils.json import object_hook


class CustomTask(Task):
    def __call__(self, *args, **kwargs):
        def _t(arg):
            if hasattr(arg, 'transform'):
                return arg.transform()
            return arg

        return self.run(*[_t(a) for a in args], **kwargs)


def serialize(obj):
    if hasattr(obj, '__json__'):
        obj = obj.__json__()
    return json.dumps(obj, check_circular=False)


def deserialize(obj):
    return json.loads(
        obj, _loads=functools.partial(
            json.json.loads, object_hook=object_hook))


# Register the custom serializer with kombu
register('girder_io', serialize, deserialize,
         content_type='application/json',
         content_encoding='utf-8')

app = Celery('proj',
             broker='amqp://',
             backend='amqp://',
             include=['gw_utils_demo_app.tasks', 'girder_worker_utils.transforms.girder'],
             task_cls=CustomTask)

app.conf.update(
    accept_content=['girder_io'],
    result_expires=3600,
    task_serializer='girder_io',
    result_serializer='girder_io',
    worker_send_task_events=True,
    task_send_sent_event=True

)

if __name__ == '__main__':
    app.start()



import json

from girder_worker_utils.transform import Transform
from girder_worker_utils.transforms import girder
from gw_utils_demo_app import tasks
from gw_utils_demo_app.transform import RaiseException, Reverse

api_url = 'https://data.kitware.com/api/v1'

# tasks.task.delay('foobar')
print(tasks.task(Transform('foobar')))
print(tasks.task(Reverse('foobar')))
print(tasks.task.delay('foobar').get())
print(tasks.task.delay(Reverse('foobar')).get())

print(girder.GirderFileToString('55c8998f8d777f6ddc3ff818', api_url=api_url).transform()[:35])
data = json.loads(tasks.task.delay(
    girder.GirderFileToString('55c8998f8d777f6ddc3ff818', api_url=api_url)
).get())
print(data.keys())

data = tasks.task.delay(
    girder.GirderFileToFileObject('55c8998f8d777f6ddc3ff818', api_url=api_url)
).get()
print(data)

data = tasks.task.delay(
    girder.GirderFileToLocalFile('55c8998f8d777f6ddc3ff818', api_url=api_url)
).get()
print(data)

print(tasks.task.delay(RaiseException()).get(propagate=False))
